@@ -6,16 +6,9 @@ import pandas as pd
 import ujson
 
 
-def datetime_parser(date_time: str):
-    """ datetime in iso8601 format parser """
-    a, b = date_time.split('+')
-    dt = datetime.datetime.strptime(a, "%Y-%m-%dT%H:%M:%S")
-    return dt
-
-
 class YClientsAPI:
 
-    def __init__(self, token, company_id, form_id, language='ru-RU'):
+    def __init__(self, token: str, company_id: int, form_id: int, language: str = 'ru-RU'):
         self.company_id = company_id
         self.form_id = form_id
         self.headers = {
@@ -27,10 +20,18 @@ class YClientsAPI:
         # if __show_debugging==True code will show debugging process
         self.__show_debugging = False
 
-    def book(self, booking_id, fullname, phone, email, staff_id, date_time, service_id=None, comment=None):
+    @staticmethod
+    def datetime_parser(date_time: str) -> datetime.datetime:
+        """ datetime in iso8601 format parser """
+        a, b = date_time.split('+')
+        dt = datetime.datetime.strptime(a, "%Y-%m-%dT%H:%M:%S")
+        return dt
+
+    def book(self, booking_id: int, fullname: str, phone: str, email: str, staff_id: int,
+             date_time: str | datetime.datetime,
+             service_id: int = None, comment: str = None) -> (True, '') or (False, Exception):
         """ Make booking """
         url = "https://n{}.yclients.com/api/v1/book_record/{}/".format(self.form_id, self.company_id)
-
         payload = {
             "phone": phone,
             "fullname": fullname,
@@ -44,49 +45,49 @@ class YClientsAPI:
                 "datetime": date_time
             }]
         }
-        response = httpx.request("POST", url, json=payload, headers=self.headers)
+        response = httpx.post(url, json=payload, headers=self.headers)
         res = ujson.loads(response.text)
         if isinstance(res, dict) and res.get('errors'):
             return False, res.get('errors', {}).get('message', '')
         return True, ''
 
-    def get_staff_info(self, staff_id):
+    def get_staff_info(self, staff_id: int) -> dict:
         """ Return dict with info about specific staff"""
         url = "https://n{}.yclients.com/api/v1/staff/{}/{}".format(self.form_id, self.company_id, staff_id)
-        response = httpx.request("GET", url, headers=self.headers)
+        response = httpx.get(url, headers=self.headers)
         return ujson.loads(response.text)
 
-    def get_service_info(self, service_id):
+    def get_service_info(self, service_id: int) -> dict:
         """ Return dict with info about specific service"""
         url = "https://n{}.yclients.com/api/v1/services/{}/{}".format(self.form_id, self.company_id, service_id)
-        response = httpx.request("GET", url, headers=self.headers)
+        response = httpx.get(url, headers=self.headers)
         return ujson.loads(response.text)
 
-    def get_staff(self, service_id=None, date_time=None):
-        """ Return list of staff for specific service and date"""
+    def get_staff(self, service_id: int = None, date_time=None) -> dict:
+        """ Return dict of staff for specific service and date"""
         url = "https://n{}.yclients.com/api/v1/book_staff/{}".format(self.form_id, self.company_id)
         querystring = {"service_ids[]": int(service_id)} if service_id else {}
         querystring.update({"datetime": date_time} if date_time else {})
-        response = httpx.request("GET", url, headers=self.headers, params=querystring)
+        response = httpx.get(url, headers=self.headers, params=querystring)
         return ujson.loads(response.text)
 
-    def get_services(self, staff_id=None, date_time=None):
+    def get_services(self, staff_id: int = None, date_time: int = None) -> dict:
         """ Return list of services for specific staff and date"""
         url = "https://n{}.yclients.com/api/v1/book_services/{}".format(self.form_id, self.company_id)
         querystring = {"staff_id": int(staff_id)} if staff_id else {}
         querystring.update({"datetime": date_time} if date_time else {})
-        response = httpx.request("GET", url, headers=self.headers, params=querystring)
+        response = httpx.get(url, headers=self.headers, params=querystring)
         return ujson.loads(response.text)
 
-    def get_available_days(self, staff_id=None, service_id=None):
+    def get_available_days(self, staff_id: int = None, service_id: int = None) -> dict:
         """ Return all available days for specific staff and service"""
         url = "https://n{}.yclients.com/api/v1/book_dates/{}".format(self.form_id, self.company_id)
         querystring = {"staff_id": int(staff_id)} if staff_id else {}
         querystring.update({"service_ids[]": service_id} if service_id else {})
-        response = httpx.request("GET", url, headers=self.headers, params=querystring)
+        response = httpx.get(url, headers=self.headers, params=querystring)
         return ujson.loads(response.text)
 
-    def get_available_times(self, staff_id, service_id=None, day=None):
+    def get_available_times(self, staff_id: int, service_id: int = None, day=None) -> dict:
         """ Return all available time slots on specific day staff and service"""
         url = "https://n{}.yclients.com/api/v1/book_times/{}/{}/{}".format(self.form_id, self.company_id, staff_id, day)
         querystring = {}
@@ -107,7 +108,7 @@ class YClientsAPI:
 
     """USER AUTHORIZATION"""
 
-    def get_user_token(self, login, password):
+    def get_user_token(self, login: str, password: str) -> str:
         """
         To read clients data you need to obtain user token
         :param login: yclients user login
@@ -119,13 +120,13 @@ class YClientsAPI:
             "login": login,
             "password": password
         }
-        response = httpx.request("POST", url, headers=self.headers, params=querystring)
+        response = httpx.post(url, headers=self.headers, params=querystring)
         user_token = ujson.loads(response.text)['data']['user_token']
         if self.__show_debugging:
             print(f"Obtained user token {user_token}")
         return user_token
 
-    def update_user_token(self, user_token):
+    def update_user_token(self, user_token: str):
         """
         After user token was obtained you need to include it in
         header of requests that you are sending
@@ -152,13 +153,13 @@ class YClientsAPI:
 
     """CLIENTS DATA"""
 
-    def __get_clients_page(self, page_number, session, clients_per_page):
+    def __get_clients_page(self, page_number: int, session: httpx.Client, clients_per_page: int) -> dict:
         """
         Yclients api can't return all clients at once and returns in groups
          of maximum size of 200. Those groups are called pages and you can
          choose how many clients it will return
         :param page_number: number of page
-        :param session: requests.Session() object
+        :param session: httpx.Client() object
         :param clients_per_page: size of the page
         :return: data of clients on the page page_number
         """
@@ -172,7 +173,7 @@ class YClientsAPI:
             print(f'Clients page {page_number} obtained in {time.time() - st} sec')
         return ujson.loads(response.text)
 
-    def get_clients_data(self, clients_per_page=200):
+    def get_clients_data(self, clients_per_page: int = 200) -> dict:
         """
         :param clients_per_page: size of the page
         :return: data of all the clients in the system
@@ -184,11 +185,14 @@ class YClientsAPI:
                 'categories', 'last_change_date', 'custom_fields'
         """
         session = httpx.Client(trust_env=False)
+
         # In the first request we obtain total number of clients in system
         first_request = self.__get_clients_page(1, session, clients_per_page)
         clients_data_list = first_request['data']
+
         # total number of clients
         clients_number = first_request['meta']['total_count']
+
         # number of pages that we need to request
         pages_number = int(clients_number / clients_per_page) + 1
         if self.__show_debugging:
@@ -198,19 +202,19 @@ class YClientsAPI:
 
         if pages_number == 1:
             return clients_data_list
-        else:
-            for page in range(2, pages_number + 1):
-                new_page_request = self.__get_clients_page(page, session, clients_per_page)
-                clients_data_list.extend(new_page_request['data'])
-            return clients_data_list
 
-    def parse_clients_data(self, clients_data_list: list):
+        for page in range(2, pages_number + 1):
+            new_page_request = self.__get_clients_page(page, session, clients_per_page)
+            clients_data_list.extend(new_page_request['data'])
+        return clients_data_list
+
+    def parse_clients_data(self, clients_data_list: list) -> pd.DataFrame:
         """
         :param clients_data_list: list of dictionaries with client data
         :return: pd.Dataframe with clients data
         """
-        colums = clients_data_list[0].keys()
-        df = pd.DataFrame(columns=colums)
+        columns = clients_data_list[0].keys()
+        df = pd.DataFrame(columns=columns)
         for client_data in clients_data_list:
             df = df.append(client_data, ignore_index=True)
         if self.__show_debugging:
@@ -220,7 +224,7 @@ class YClientsAPI:
 
     """VISITS DATA"""
 
-    def __get_visits_page(self, cid, page_number, session, visits_per_page):
+    def __get_visits_page(self, cid: int, page_number: int, session: httpx.Client, visits_per_page: int) -> dict:
         """
         Yclients api can't return all visits at once and returns in groups
          of maximum size of 200. Those groups are called pages and you can
@@ -243,7 +247,7 @@ class YClientsAPI:
 
         return ujson.loads(response.text)
 
-    def get_visits_for_client(self, cid, visits_per_page=200, session=None):
+    def get_visits_for_client(self, cid: int, visits_per_page: int = 200, session: httpx.Client = None) -> list:
         """
         :param cid: client id
         :param visits_per_page: size of the page
@@ -279,7 +283,7 @@ class YClientsAPI:
             Parameters description is  accesible via:
             https://github.com/petroff/api-blueprint/blob/master/apiary.apib
         """
-        if session is None:
+        if not session:
             session = httpx.Client(trust_env=False)
 
         # In the first request we obtain total number of client visits in system
@@ -299,16 +303,16 @@ class YClientsAPI:
 
         if pages_number == 1:
             return visits_data_list
-        else:
-            for page in range(2, pages_number + 1):
-                new_page_request = self.__get_visits_page(cid=cid,
-                                                          page_number=page,
-                                                          session=session,
-                                                          visits_per_page=visits_per_page)
-                visits_data_list.extend(new_page_request['data'])
-            return visits_data_list
 
-    def get_visits_data_for_clients_list(self, cids_list, visits_per_page=200):
+        for page in range(2, pages_number + 1):
+            new_page_request = self.__get_visits_page(cid=cid,
+                                                      page_number=page,
+                                                      session=session,
+                                                      visits_per_page=visits_per_page)
+            visits_data_list.extend(new_page_request['data'])
+        return visits_data_list
+
+    def get_visits_data_for_clients_list(self, cids_list: list, visits_per_page=200) -> dict:
         """
         get_visits_for_client funtion wrapper for multiple clients
         :param cids_list: list of clients ids
@@ -317,15 +321,12 @@ class YClientsAPI:
         """
         session = httpx.Client(trust_env=False)
 
-        clients_visits_dictionary = {}
-        for cid in cids_list:
-            visits = self.get_visits_for_client(cid=cid,
-                                                visits_per_page=visits_per_page,
-                                                session=session)
-            clients_visits_dictionary[cid] = visits
+        clients_visits_dictionary = {cid: self.get_visits_for_client(cid, visits_per_page, session)
+                                     for cid in cids_list}
         return clients_visits_dictionary
 
-    def get_attended_visits_for_client(self, cid, visits_per_page=200, session=None):
+    def get_attended_visits_for_client(self, cid: int, visits_per_page: int = 200,
+                                       session: httpx.Client = None) -> list:
         """
         Attendance explanation from Yclient API:
             2 - The user has confirmed the entry,
@@ -341,13 +342,12 @@ class YClientsAPI:
         all_visits = self.get_visits_for_client(cid=cid,
                                                 visits_per_page=visits_per_page,
                                                 session=session)
-        attended_visits = []
-        for visit in all_visits:
-            if visit['attendance'] == 1:
-                attended_visits.append(visit)
+        attended_visits = [visit for visit in all_visits if visit['attendance'] == 1]
+
         return attended_visits
 
-    def get_attended_visits_dates_information(self, cids_lists, visits_per_page=200, session=None):
+    def get_attended_visits_dates_information(self, cids_lists: list, visits_per_page: int = 200,
+                                              session: httpx.Client = None) -> pd.DataFrame:
         """
         :param cids_lists: clients ids list
         :param visits_per_page: size of the page
@@ -367,18 +367,17 @@ class YClientsAPI:
                                                                   visits_per_page=visits_per_page,
                                                                   session=session
                                                                   )
-            visit_dates = []
-            for visit in attended_visits:
-                visit_dates.append(visit['datetime'])
+            visit_dates = [visit['datetime'] for visit in attended_visits]
 
             if not visit_dates:
                 c_dict['visits_number'] = 0
                 c_dict['first_visit'] = None
                 c_dict['last_visit'] = None
             else:
-                visit_dates = list(map(datetime_parser, visit_dates))
+                visit_dates = list(map(self.datetime_parser, visit_dates))
                 c_dict['visits_number'] = len(visit_dates)
                 c_dict['first_visit'] = min(visit_dates).date()
                 c_dict['last_visit'] = max(visit_dates).date()
+
             df = df.append(c_dict, ignore_index=True)
         return df
